﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using UnityEngine.SceneManagement;  // シーンの切り替えに使用

//////////////////////////////////////////////////////////////////
//【ゲーム全体の管理に使用するクラス】
// ゲームオーバー・ゲームクリア・リスタート処理を実行
// コンティニュー機能、敵を定期的に生成
//////////////////////////////////////////////////////////////////
public class GameController : MonoBehaviour
{
    /////////////////////////////////
    // 【変数の設定】
    /////////////////////////////////

    //【定数の設定】（レベル設計で変更可能）
    const float CONTINUE_LV1 = 1.5f;  // コンティニュー用の１つ目の通過ポイント
    const float CONTINUE_LV2 = 7.5f;  // コンティニュー用の２つ目の通過ポイント
    const float ENEMY_CREATE_TIME = 10;  // 敵を定期的に再生成するタイミング

    //【Unity Editorから設定するPublic変数】
    public GameObject text_GameOver;  // ゲームオーバーのUIテキスト
    public GameObject text_GameClear;  // ゲームクリアのUIテキスト

    public GameObject player;  // コンティニュー機能用にPlayerの位置取得用
    public GameObject prefab_Enemy;  // 再生成用の敵のPrefab
    public AudioSource audioSource;  // サウンド制御用のAudioSource

    public bool stopGame = false;  // ゲーム終了ステイタス(他クラスからも使用)


    /////////////////////////////////
    // 【関数の設定】
    /////////////////////////////////

    //【ゲーム開始時に１回だけ動く関数】
    //（Unityの標準関数）
    void Start()
    {
        // 保存してあるコンティニュー用レベルを取得する
        switch (PlayerPrefs.GetInt("CONT_LV", 1))
        {
            // 保存してあるコンティニュー用レベルが１の場合
            case 1:
                // Playerを指定位置に移動させる
                player.transform.position = new Vector3(-8, -4, 0);
                break;

            // 保存してあるコンティニュー用レベルが２の場合
            case 2:
                // Playerを指定位置に移動させる
                player.transform.position = new Vector3(-7, CONTINUE_LV1 + 1.5f, 0);
                break;

            // 保存してあるコンティニュー用レベルが３の場合
            case 3:
                // Playerを指定位置に移動させる
                player.transform.position = new Vector3(-7, CONTINUE_LV2 + 1.5f, 0);
                break;
        }

        // 敵を繰り返し発生させるコルーチンを呼ぶ
        StartCoroutine(Create_Enemy());
    }


    //【ゲームオーバー時に他のプログラムから呼ばれるゲームオーバー処理の関数】
    public void On_GameOver()
    {
        // ゲームオーバーのUIテキストを表示
        text_GameOver.SetActive(true);

        // 再生中のサウンドを停止
        audioSource.Stop();

        // ゲーム終了ステイタスを終了にする
        stopGame = true;

        int continue_Level = 1;  // コンティニュー用レベル変数を1で初期化
        // ゲームオーバー時にPlayerのY座標が2つ目の通過ポイントより上の場合
        if (player.transform.position.y > CONTINUE_LV2)
            // コンティニュー用レベル変数を3に
            continue_Level = 3;

        // ゲームオーバー時にPlayerのY座標が1つ目の通過ポイントより上の場合
        else if (player.transform.position.y > CONTINUE_LV1)
            // コンティニュー用レベル変数を2に
            continue_Level = 2;

        // コンティニュー用レベルを保存する
        PlayerPrefs.SetInt("CONT_LV", continue_Level);
    }


    //【ゲームクリア時に他のプログラムから呼ばれるゲームクリア処理の関数】
    public void On_GameClear()
    {
        // ゲームクリアのUIテキストを表示
        text_GameClear.SetActive(true);

        // 再生中のサウンドを停止
        audioSource.Stop();

        // ゲーム終了ステイタスを終了にする
        stopGame = true;
    }


    //【リスタート用ボタンを押された際の処理関数】
    public void OnClick_Restart()
    {
        // GameMainシーンを呼び出す
        SceneManager.LoadScene("GameMain");
    }


    //【敵を発生させるコルーチン】
    IEnumerator Create_Enemy()
    {
        // プログラムを指定秒停止させる
        yield return new WaitForSeconds(ENEMY_CREATE_TIME);

        // 敵のプレハブをシーン上に作成する
        Instantiate(prefab_Enemy, new Vector3(0, player.transform.position.y + 1, 0),
            Quaternion.identity);

        // ゲームが終了してない場合、敵を繰り返し発生させるコルーチンを再度呼ぶ
        if (stopGame == false) StartCoroutine(Create_Enemy());
    }
}
